if (navigator.serviceWorker) {
    initServiceWorker();
}

async function initServiceWorker() {
    let swRegistration = await navigator.serviceWorker.register('/serviceworker.js', {scope: ''})
    let pushManager = swRegistration.pushManager;
    if (!isPushManagerActive(pushManager)) {
        return;
    }
    if(isStandAlone()) {
        let permissionState = await pushManager.permissionState({userVisibleOnly: true});
        switch (permissionState) {
            case 'prompt':
                document.getElementById('request-webpush').style.display = 'block';
                // document.getElementById('pwa-guide').style.display = 'none';
                break;
            case 'granted':
                displaySubscriptionInfo(await pushManager.getSubscription())
                break;
            case 'denied':
            // document.getElementById('request-webpush').style.display = 'none';
            // document.getElementById('denied-webpush').style.display = 'block';
        }
    }
    return;

}

function isPushManagerActive(pushManager) {
    if (!pushManager) {
        return false;
    } else {
        return true;
    }
}

function isStandAlone(){
    try {
        if (!window.navigator.standalone) {
            document.getElementById('pwa-guide').style.display = 'block';
            document.getElementById('request-webpush').style.display = 'none';
            return false
        } else {
            return true
        }

    } catch (e){
        try {
            document.getElementById('pwa-guide').style.display = 'block';
            document.getElementById('request-webpush').style.display = 'none';
        }catch(e) {
            
        }
        return false
    }
}

function urlBase64ToUint8Array(base64String) {
    const padding = '='.repeat((4 - (base64String.length % 4)) % 4);
    const base64 = (base64String + padding).replace(/\-/g, '+').replace(/_/g, '/');

    const rawData = window.atob(base64);
    const outputArray = new Uint8Array(rawData.length);

    for (let i = 0; i < rawData.length; ++i) {
        outputArray[i] = rawData.charCodeAt(i);
    }
    return outputArray;
}

async function subscribeToPush() {
    try {
        // Public part of VAPID key, generation of that covered in README
        // All subscription tokens associated with that key, so if you change it - you may lose old subscribers
        const VAPID_PUBLIC_KEY = 'BEJNvGycgoJVFMDzoVB09ddc5OJ6cZSR0cDkIaY7WVLnbiA0Wz2csPzlI1ozktfARQQ6TnF_MR69mp4TG9xAI_c';
        let swRegistration = await navigator.serviceWorker.register('/serviceworker.js', {scope: ''})
        // let swRegistration = await navigator.serviceWorker.getRegistration();
        let pushManager = swRegistration.pushManager;
        if (!isPushManagerActive(pushManager)) {
            return;
        }
        let subscriptionOptions = {
            userVisibleOnly: true,
            applicationServerKey: urlBase64ToUint8Array(VAPID_PUBLIC_KEY)
        };
        try {
            let subscription = await pushManager.subscribe(subscriptionOptions);
            displaySubscriptionInfo(subscription);
            // window.location.href = '/'
            // Here you can send fetch request with subscription data to your backend API for next push sends from there
        } catch (error) {
            document.getElementById('denied-webpush').style.display = 'block';
            // window.location.href = '/'
        }
    } catch (e) {
        console.log(e)
        // window.location.href = '/'
    }
}

function displaySubscriptionInfo(subscription) {
    window._APP_WEBPUSH_TOKEN = btoa(JSON.stringify(subscription.toJSON()))
    try {
        document.getElementById('request-webpush').style.display = 'none';
        document.getElementById('debug-subcription').style.display = 'block';
        document.getElementById('debug-subcription').innerText = JSON.stringify(subscription.toJSON());

    } catch (e) {
        // alert("ERROR")
    }
}



function testSend() {
    const title = "The Kevin Push Test";
    const options = {
        body: "Push test body is here",
        // icon: "https://andreinwald.github.io/webpush-ios-example/images/push_icon.jpg",
        // image: "https://upload.wikimedia.org/wikipedia/commons/thumb/6/68/Orange_tabby_cat_sitting_on_fallen_leaves-Hisashi-01A.jpg/1920px-Orange_tabby_cat_sitting_on_fallen_leaves-Hisashi-01A.jpg",
        // data: {
        //     "url": "https://andreinwald.github.io/webpush-ios-example/success.html",
        //     "message_id": "your_internal_unique_message_id_for_tracking"
        // },
    };
    navigator.serviceWorker.ready.then(async function (serviceWorker) {
        await serviceWorker.showNotification(title, options);
    });
}
